// This file is part of Eigen, a lightweight C++ template library
// for linear algebra. Eigen itself is part of the KDE project.
//
// Copyright (C) 2006 Benoit Jacob <jacob@math.jussieu.fr>
//
// Eigen is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 or (at your option) any later version.
//
// Eigen is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along
// with Eigen; if not, write to the Free Software Foundation, Inc., 51
// Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. This exception does not invalidate any other reasons why a work
// based on this file might be covered by the GNU General Public License.

#include "main.h"

using namespace std;

void MainTest::checkLinearRegression()
{
    for( int repeat = 0; repeat < 300; repeat++ )
    {
        Vector2f points2f [1000];
        Vector3f coeffs3f;
        makeNoisyCohyperplanarPoints< float, 2, Vector2f, Vector3f >
            ( 1000, points2f, &coeffs3f, 0.01 );
        helper_checkComputeFittingHyperplane
                    < float, Vector2f, Vector3f, Matrix2f >
                    ( 10, points2f, &coeffs3f, 0.05 );
        helper_checkComputeFittingHyperplane
                    < float, Vector2f, Vector3f, Matrix2f >
                    ( 100, points2f, &coeffs3f, 0.01 );
        helper_checkComputeFittingHyperplane
                    < float, Vector2f, Vector3f, Matrix2f >
                    ( 1000, points2f, &coeffs3f, 0.002 );

        Vector4d points4d [1000];
        Vector<double, 5> coeffs5d;
        makeNoisyCohyperplanarPoints< double, 4, Vector4d, Vector<double,5> >
            ( 1000, points4d, &coeffs5d, 0.01 );
        helper_checkComputeFittingHyperplane
                    < double, Vector4d, Vector<double, 5>, Matrix4d >
                    ( 10, points4d, &coeffs5d, 0.05 );
        helper_checkComputeFittingHyperplane
                    < double, Vector4d, Vector<double, 5>, Matrix4d >
                    ( 100, points4d, &coeffs5d, 0.01 );
        helper_checkComputeFittingHyperplane
                    < double, Vector4d, Vector<double, 5>, Matrix4d >
                    ( 1000, points4d, &coeffs5d, 0.002 );

        VectorXcd points8cd [1000];
        VectorXcd coeffs9cd;
        makeNoisyCohyperplanarPoints< complex<double>, 8, VectorXcd, VectorXcd >
            ( 1000, points8cd, &coeffs9cd, 0.01 );
        helper_checkComputeFittingHyperplane
                    < complex<double>, VectorXcd, VectorXcd, MatrixXcd >
                    ( 10, points8cd, &coeffs9cd, 0.35 );
        helper_checkComputeFittingHyperplane
                    < complex<double>, VectorXcd, VectorXcd, MatrixXcd >
                    ( 100, points8cd, &coeffs9cd, 0.025 );
        helper_checkComputeFittingHyperplane
                    < complex<double>, VectorXcd, VectorXcd, MatrixXcd >
                    ( 1000, points8cd, &coeffs9cd, 0.006 );

        VectorXd points30d [1000];
        VectorXd coeffs31d;
        makeNoisyCohyperplanarPoints< double, 30, VectorXd, VectorXd >
            ( 1000, points30d, &coeffs31d, 0.01 );
        helper_checkComputeFittingHyperplane
                    < double, VectorXd, VectorXd, MatrixXd >
                    ( 100, points30d, &coeffs31d, 0.1 );
        helper_checkComputeFittingHyperplane
                    < double, VectorXd, VectorXd, MatrixXd >
                    ( 1000, points30d, &coeffs31d, 0.015 );
    }
}

void MainTest::checkLinearSolver()
{

}

QTEST_APPLESS_MAIN( MainTest )
#include "main.moc"
